#include "EmbededRobotLibrary.h"
#include "i2c_bno055.h"
#include "timestamp.h"

BNO055  sys_imu(0x28);

Timestamp ts;

void setup()
{
  Wire.begin(21, 22, 400000);
  Serial.begin(115200);

  pinMode(13, OUTPUT);                // Error flag pin
  pinMode(2,  OUTPUT);                // Activity LED

  delay(1500);

  init_imu();
}

void loop()
{
  static int          loop_count;
  static EulerVector  prev_euler;

  uint                print_count = 100;
  EulerVector         euler;

  double  h_delta;
  double  p_delta;
  double  r_delta;
  double  total_delta;

  Serial.print("Start loop.\n");
  while(1)
  {
    digitalWrite(2, HIGH);
    sys_imu.update();

    if (sys_imu.rx_errs or sys_imu.tx_errs)
    {
      int i2c_error = sys_imu.getLastError();
      digitalWrite(13, LOW);
      Serial.print(ts.secs());
      Serial.print("!  txe: ");
      Serial.print(sys_imu.tx_errs);
      Serial.print("  rxe: ");
      Serial.print(sys_imu.rx_errs);
      Serial.print("  sle: ");
      Serial.println(i2c_error);

      sys_imu.rx_errs = 0;
      sys_imu.tx_errs = 0;
      init_imu();
      delay(1000);
    }
    else
    {
      euler = sys_imu.get();
      h_delta      = abs(prev_euler.heading - euler.heading);
      p_delta      = abs(prev_euler.pitch - euler.pitch);
      r_delta      = abs(prev_euler.roll - euler.roll);
      total_delta  = h_delta + p_delta + r_delta;

      if (total_delta > 2.0)
      {
        Serial.print(ts.secs());
        Serial.print("^ ");
        Serial.print(sys_imu.getLastError());
        Serial.print("  H ");
        Serial.print((int)(euler.heading + 0.5));
        Serial.print("  P ");
        Serial.print((int)(euler.pitch + 0.5));
        Serial.print("  R ");
        Serial.println((int)(euler.roll + 0.5));
        prev_euler = euler;
      }

      if (print_count == 100)
      {
        Serial.print(ts.secs());
        Serial.print("  ");
        Serial.print(sys_imu.getLastError());
        Serial.print("  H ");
        Serial.print(euler.heading);
        Serial.print("  P ");
        Serial.print(euler.pitch);
        Serial.print("  R ");
        Serial.print(euler.roll);
        Serial.print("  T ");
        Serial.println(sys_imu.getTemperature());
        print_count = 0;
      }

      print_count++;
      loop_count++;

      delay(5);
      digitalWrite(2, LOW);
      delay(45);
    }
  }
}

void init_imu()
{
  bool    ping = false;

  ping = sys_imu.ping();
  Serial.print("    Ping:           ");
  Serial.println(ping);

  while (!ping)
  {
    delay(250);
    ping = sys_imu.ping();
    Serial.print('.');
  }

  Serial.print("    BNO055 Present: ");
  Serial.println(sys_imu.isPresent());
  digitalWrite(13, HIGH);

  // Put IMU in configuration mode.
  sys_imu.enable(true);
  sys_imu.setOprMode(sys_imu.OPR_MODE_CFG);
  sys_imu.setDataSource(sys_imu.SOURCE_EULER);
  sys_imu.setTempUnits(sys_imu.TEMP_UNIT_C);

  // Put IMU in fusion mode.
  sys_imu.setOprMode(sys_imu.OPR_MODE_IMU);

  // Read IMU configuration values.
  Serial.print("    Software Rev:   ");
  Serial.println(sys_imu.getSoftwareRev());
  Serial.print("    Boot Rev:       ");
  Serial.println(sys_imu.getBootRev());
  Serial.print("    Self Test:      ");
  Serial.println(sys_imu.getSelfTest(), HEX);
  Serial.print("    Calibration:    ");
  Serial.println(sys_imu.getCalStatus(), HEX);
  Serial.print("    Error Status:   ");
  Serial.println(sys_imu.getSysError());
  Serial.print("    System Status:  ");
  Serial.println(sys_imu.getSysStatus());
  Serial.print("    Units Register: ");
  Serial.println(sys_imu.getUnits(), HEX);
  Serial.print("    Data Source:    ");
  Serial.println(sys_imu.getDataSource());
  Serial.print("    Operation Mode: ");
  Serial.println(sys_imu.getOprMode());
}
