/*
 * i_i2c.h
 *
 * Copyright (C) 2013 Kyle Crane
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef EMBEDEDROBOTLIBRARY_I_I2C_H
#define EMBEDEDROBOTLIBRARY_I_I2C_H

// I2C Error codes for errno inspection.
enum I2CERR
{
  I2C_OK       = 0,   /** No error                 */
	ERR_I2C_GEN	 = -1,  /** Unspecified error        */
	ERR_I2C_BSY  = -2,  /** Bus is busy              */
	ERR_I2C_IO	 = -4,  /** General I/O error on bus */
	ERR_I2C_RNG  = -5,  /** Input range error        */
	ERR_I2C_NRDY = -6,  /** I2C not ready            */
};


/** @brief Interface definition for objects that access the I2C bus.
 *
 */
class I_I2C
{
public:
  virtual ~I_I2C(){};

public:
  // Required overrides:

  /** Return true if device address is found on the bus    */
  virtual bool ping()=0;

  /** Read data from I2C bus from register to buffer       */
  virtual bool read(uint16_t reg, uint8_t* buffer, uint count)=0;

  /** Write data to the I2C device using register address.  */
  virtual bool write(uint16_t reg, uint8_t* byte, uint count)=0;

  /** Returns any error generated by the last  operation    */
  virtual int  getLastError()=0;
};


#endif
