/* 
 * File:   ifilter.h
 * Author: kcrane
 *
 * Created on September 9, 2014, 9:58 AM
 */

#ifndef IFILTER_H
#define	IFILTER_H

#include <stdint.h>

/** @brief Generic interface to all digital filters 
 * 
 */
template <class T>
class IFilter 
{
    public:
    virtual ~IFilter() {};
    
    /**@brief Provide a sample input to the filter.
     * 
     * @param T sample:   The sample input to the filter.
     * @return T:         Return the latest filtered output.
     */
    virtual T in(T sample)=0;
    
    /**@brief Get the current filtered value without providing a sample
     * 
     * @return T:        Return the latest filtered output.
     */
    virtual T out()=0;
    
    /** @brief Clear all internal data and state.  
     *
     * Return filter to its initial state.
     */
    virtual void clear()=0;
    
    /** @brief Set the number of samples used to compute the filter value
     * 
     *  Optional override to allow the user to reset the number of samples
     *  the filter will use to create it's filtered output.
     * 
     *  @param uint32_t count:  Number of samples to use. 
     */
    virtual void setSampCount(uint32_t count) {};
    
    protected:
        T filterValue;
};



#endif	/* IFILTER_H */

