/*****************************************************************************/
/*                                isensor.h                                  */
/*****************************************************************************/

#ifndef ISENSOR_H_
#define ISENSOR_H_

#include <WString.h>

template <class RET_TYPE> class ISensor
{
public:
  const unsigned FORCE = 1;

public:
  virtual ~ISensor()
  {
    delete m_pFilter;
  };

  //
  // Required overrides:
  //
  /** @brief Update sensor internal data            */
  virtual void update()=0;

  /** @brief Return a result type from the sensor.  */
  virtual RET_TYPE  get(unsigned force=0)=0;


  //
  // Optional overrides
  //
  virtual void set_name(const char name[])
  {m_name = name;};

  virtual String get_name(void)
  {return m_name;};

  /** @brief Enable the sensor.                     */
  virtual void enable(bool ena = true)   { m_enable = ena; } ;

  /** @brief Return the enable state of the sensor  */
  virtual bool isEnabled() { return m_enable; };

  /** @brief Set the sensor's filter object */
  virtual void setFilter(IFilter<RET_TYPE>* pFilter)
  {
    if (pFilter)
    {
      delete m_pFilter;
      m_pFilter = pFilter;
    }
    else
    {
      m_pFilter = 0;
    }
  };

  virtual void toString(String& output, int detail=0)
  {output = "Unimplemented";};

protected:
  String m_name;                /** Sensor name               */
  bool   m_enable;              /** Object enable flag        */
  IFilter<RET_TYPE>* m_pFilter; /** Sensor result-filter slot */
};


///////////////////////////////////////////////////////////////////////////////
// Simple Structures to return multiple values
//
// These structures can be used to return results from sensors that provide
// more than a single value when queried.
//
struct SenDblInt
{
  int x;
  int y;
};

struct SenDblFloat
{
  double x;
  double y;
};

struct SenTriInt
{
  int x;
  int y;
  int z;
};

struct SenTriFloat
{
  double x;
  double y;
  double z;
};

struct SenIntFloat
{
  int      x;
  double   y;
};

/*
 Copyright (C) 2016 Kyle Crane

 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to
 use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 of the Software, and to permit persons to whom the Software is furnished to do
 so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 */

#endif /* SENSORS_ISENSOR_H_ */
