/* 
 * File:   UnityFilter.h
 * Author: kcrane
 *
 * Created on September 9, 2014, 9:44 PM
 */

#ifndef UNITYFILTER_H
#define	UNITYFILTER_H

#include "ifilter.h"

/** @brief Class represents a unity filter that directly follows input values
 * 
 *  The unity filter is a default filter that can be used as a place-holder 
 *  filter.  The output is always exactly equal to the input.   
 */
template <class T>
class UnityFilter : public IFilter<T>
{
public:
    
    /** Construct an empty UnityFilter object.
     * 
     */
    UnityFilter()
    {
        this->filterValue = 0;
    };
    
    
    /** @brief Default destructor.
     * 
     */
    virtual ~UnityFilter() {};
    
    
    /** @brief Apply the sample to the filter and return the new filter value.
     * 
     * @param T sample:     The sample value to input to the filter.
     * @return T:           The current output value of the filter.
     */
    virtual T in(T sample)
    {
        this->filterValue = sample;
        return this->filterValue;
    };
    
    
    /** @brief Return the current filter output value.
     * 
     * @return 
     */
    virtual T out()
    {
        return this->filterValue;
    };
    
    
    /** @brief Clear the filter output value.
     * 
     */
    virtual void clear()
    {
       this->filterValue = 0; 
    };
    
    
private:

};

#endif	/* UNITYFILTER_H */

